@@ -80,8 +80,6 @@ const gchar *empathy_service_name_to_display_name (const gchar *proto_name);
 #define EMPATHY_ARRAY_TYPE_OBJECT (empathy_type_dbus_ao ())
 GType empathy_type_dbus_ao (void);
 
-TpAccount * empathy_get_account_for_connection (TpConnection *connection);
-
 gboolean empathy_account_manager_get_accounts_connected (gboolean *connecting);
 
 void empathy_connect_new_account (TpAccount *account,
@@ -108,6 +106,8 @@ gchar *empathy_format_currency (gint amount,
     guint scale,
     const gchar *currency);
 
+gboolean empathy_account_has_uri_scheme_tel (TpAccount *account);
+
 /* Copied from wocky/wocky-utils.h */
 
 #define empathy_implement_finish_void(source, tag) \
